/*
 * This file is part of the SOAFaces Project (hereinafter "SOAFaces").
 * Copyright (C) 2006 Grand Logic, Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package org.soafaces.bundle.client.ui;

import org.soafaces.bundle.client.rpc.FetchPOJOCallback;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.IsSerializable;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.RootPanel;
import org.soafaces.bundle.client.rpc.FetchTaskletStatusCallback;
import org.soafaces.bundle.client.rpc.TaskletStatus;

/**
 * A convenient subclass of InputViewer. Serializes the input JavaBean associated
 * with the InputViewer using standard GWT serialization and presents it to the InputViewer GWT widget
 * as an IsSerializable object.
 *
 * <p>
 * A convenient subclass for viewing the Tasklet's runtime input
 * and output JavaBeans. This class serializes the input and output
 * JavaBeans associated with a Tasklet using standard GWT serialization
 * and presents them to the BaseIOViewer GWT client
 * as <code>IsSerializable</code> objects.
 * </p>
 *
 * <p>
 * Use this class if your Tasklet has both an input and output JavaBean
 * and if you intend to have the UI view both the input and output JavaBeans.
 * </p>
 *
 * 
 * @author Sam Taha
 */
public abstract class SimplePOJORuntimeViewer extends RuntimeViewer {
  private IsSerializable inputBean=null;
  private IsSerializable outputBean=null;
  private boolean uiLoaded=false;
  
  /**
   * Convenient implementation to return the top level UI container.
   * @return
   */
  public Panel getMainPanel() { return RootPanel.get(); }
  
  /**
   * Convenient implementation that pulls both the input and output JavaBean from the server
   * and serializes them to IsSerializable.
   */
  public final void buildUI()
  {  
    if(uiLoaded)
      return;
    uiLoaded=true;

    FetchPOJOCallback callback = new FetchPOJOCallback () {
      
      public void onFetchPOJOSuccess(IsSerializable bean)
      {
        inputBean=bean;

        FetchPOJOCallback callback = new FetchPOJOCallback ()
        {
          public void onFetchPOJOSuccess(IsSerializable bean)
          {
            outputBean=bean;

            FetchTaskletStatusCallback statusCallback = new FetchTaskletStatusCallback()
            {
              public void onFetchTaskletStatusSuccess(TaskletStatus result)
              {
                buildUI(inputBean, outputBean, result);
              }
              public void onFailure(Throwable ex)
              {
                Window.alert("Error: Problem fetching TaskletStatus from server: " + ex.getMessage());
              }
            };
            getContext().fetchTaskletStatus(statusCallback);
          }

          public void onFailure(Throwable ex)
          {
            Window.alert("Error: Problem fetching output javabean from server: " + ex.getMessage());
          }
        };
        getContext().fetchOutputBean(callback);
      } 
      
      public void onFailure(Throwable ex)
      {
        Window.alert("Error: Problem fetching input javabean from server: " + ex.getMessage());
      }
    };
    getContext().fetchInputBean(callback);
  }
  
  /**
   * Method to implement in subclass. Use this method to populate the UI and
   * setup action handlers and such.
   * 
   * @param inputBean Input JavaBean
   * @param outputBean Output JavaBean
   * @param taskletStatus Contains current status information of currently running or completed Tasklet
   */
  protected abstract void buildUI(IsSerializable inputBean, IsSerializable outputBean, TaskletStatus taskletStatus);
}